@@ -506,7 +506,7 @@ struct term {
   int marg_left, marg_right; /* horizontal margins */
   bool lrmargmode;           /* enable horizontal margins */
   bool dim_margins;
-  bool attr_rect;            /* rectangular attribute change extent */
+  uint attr_rect;            /* rectangular attribute change extent */
   bool printing, only_printing;  /* Are we doing ANSI printing? */
   int  print_state;       /* state of print-end-sequence scan */
   char *printbuf;         /* buffered data for printer */
@@ -567,6 +567,7 @@ struct term {
   bool readline_mouse_3;
 
   bool sixel_display;        // true if sixel scrolling mode is off
+  bool image_display;        // true if image scrolling is disabled
   bool sixel_scrolls_right;  // on: sixel scrolling leaves cursor to right of graphic
                              // off(default): the position after sixel depends on sixel_scrolls_left
   bool sixel_scrolls_left;   // on: sixel scrolling moves cursor to beginning of the line
@@ -691,8 +692,8 @@ extern struct term term;
 
 extern void scroll_rect(int topline, int botline, int lines);
 
-extern void term_resize(int, int);
-extern void term_scroll(int, int);
+extern void term_resize(int rows, int cols, bool quick_reflow);
+extern void term_scroll(int relative_to, int where);
 extern void term_reset(bool full);
 extern void term_clear_scrollback(void);
 extern bool term_mouse_click(mouse_button, mod_keys, pos, int count);
